// lscp_parser.c

#include "lscp_parser.h"


//-------------------------------------------------------------------------
// Simple token parser.

#if defined(WIN32)
char *strtok_r ( char *pchBuffer, const char *pszDelim, char **ppch )
{
    char *pszToken;

    if (pchBuffer == NULL)
        pchBuffer = *ppch;

    pchBuffer += strspn(pchBuffer, pszDelim);
    if (*pchBuffer == '\0')
        return NULL;

    pszToken  = pchBuffer;
    pchBuffer = strpbrk(pszToken, pszDelim);
    if (pchBuffer == NULL) {
        *ppch = strchr(pszToken, '\0');
    } else {
        *pchBuffer = '\0';
        *ppch = pchBuffer + 1;
    }

    return pszToken;
}
#endif


void lscp_parser_init ( lscp_parser_t *pParser, const char *pchBuffer, int cchBuffer )
{
    memset(pParser, 0, sizeof(lscp_parser_t));

    pParser->pchBuffer = (char *) malloc(cchBuffer + 1);
    if (pParser->pchBuffer) {
        memcpy(pParser->pchBuffer, pchBuffer, cchBuffer);
        pParser->pchBuffer[cchBuffer] = (char) 0;
        pParser->pszToken = strtok_r(pParser->pchBuffer, " \t\r\n", &(pParser->pch));
    }

}

const char *lscp_parser_next ( lscp_parser_t *pParser )
{
    const char *pszToken = pParser->pszToken;

    if (pParser->pszToken)
        pParser->pszToken = strtok_r(NULL, " \t\r\n", &(pParser->pch));

    return pszToken;
}

int lscp_parser_nextint ( lscp_parser_t *pParser )
{
    int ret = 0;

    if (pParser->pszToken) {
        ret = atoi(pParser->pszToken);
        lscp_parser_next(pParser);
    }

    return ret;
}

float lscp_parser_nextnum ( lscp_parser_t *pParser )
{
    float ret = 0;

    if (pParser->pszToken) {
        ret = atof(pParser->pszToken);
        lscp_parser_next(pParser);
    }

    return ret;
}

int lscp_parser_test ( lscp_parser_t *pParser, const char *pszToken )
{
    int ret = (pParser->pszToken != NULL);
    if (ret)
        ret = (strcmp(pParser->pszToken, pszToken) == 0);
    if (ret)
        lscp_parser_next(pParser);

    return ret;
}

int lscp_parser_test2 ( lscp_parser_t *pParser, const char *pszToken, const char *pszToken2 )
{
    int ret = lscp_parser_test(pParser, pszToken);
    if (ret)
        ret = lscp_parser_test(pParser, pszToken2);

    return ret;
}

void lscp_parser_free ( lscp_parser_t *pParser )
{
    if (pParser->pchBuffer)
        free(pParser->pchBuffer);
    pParser->pchBuffer = NULL;
}


// end of lscp_parser.c
